@@ -5,10 +5,16 @@ from .seq_traverse import *
 
 __all__ = [
     # higher order functions
+    "lazymap",
+    "lazyfilter",
+    "map",
+    "filter",
     "flip",
     "foldr",
     "comp",
     "curry",
+    "reduce",
+    "reduced",
     "trampoline",
     "threadf",
     "threadl",
@@ -31,6 +37,7 @@ __all__ = [
     "split_with",
     "count",
     "count_",
+
     # sequence transforming functions
     "cons",
     "conj",
@@ -42,6 +49,8 @@ __all__ = [
     "flatten",
     "insert",
     "remove",
+    "tuplize",
+
     # predicates
     "identity",
     "equals",
