@@ -1,9 +1,114 @@
-from typing import Callable, Any, Iterable, Union
+from typing import Callable, Any, Iterable, Union, Tuple, Generator
 from inspect import signature
 from functools import partial
 from .seq_traverse import iter_, reversed_
 
 
+class lazymap:
+    """Similar to python's map but returns a new generator
+    everytime ``__iter__`` is called on it.
+    Prefer using `functionali.map` unless you know what you're doing.
+
+
+    >>> res = lazymap(lambda x:x+1, range(100))
+    >>> take(5, res)
+    (1, 2, 3, 4, 5)
+    >>> take(5, res)
+    (1, 2, 3, 4, 5) # same result
+
+    Compare this to python's implementation
+
+    >>> res = map(lambda x:x+1, range(100))
+    >>> take(5, res)
+    (1, 2, 3, 4, 5)
+    >>> take(5, res)
+    (6, 7, 8, 9, 10)
+
+
+    Also ``__repr__`` is implemented to make repl-driven development easy :)
+
+    >>> lazymap(lambda x:x+1,range(10))
+    (1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
+    """
+
+    def __init__(self, fn: Callable, *iterables: Iterable) -> None:
+        self.fn = fn
+        self.iterables = iterables
+
+    def __iter__(self) -> Generator:
+        return (self.fn(*i) for i in zip(*self.iterables))
+
+    def __next__(res):
+        return next(res)
+
+    def __repr__(self): # pragma: no cover
+        return str(tuple(self))
+
+
+def map(fn: Callable, *iterables: Iterable) -> Tuple:
+    """Map with eager evaluation.
+    Prefer this over ``lazymap``.
+
+
+    >>> map(lambda x:x+1, range(10))
+    (1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
+    """
+    return tuple([fn(*i) for i in zip(*iterables)])
+
+
+class lazyfilter:
+    """Similar to python's filter but returns a new generator
+    everytime ``__iter__`` is called on it.
+    Prefer using `functionali.filter` unless you know what you're doing.
+
+    >>> res = lazyfilter(is_even, range(10))
+    >>> take(2, res)
+    (0, 2)
+    >>> take(2, res)
+    (0, 2) # same result
+
+    Compare this to python's implementation
+
+    # you get a different result everytime
+    >>> res = filter(is_even, range(10))
+    >>> take(2, res)
+    (0, 2)
+    >>> take(2, res)
+    (4, 6)
+    >>> take(2, res)
+    (8,)
+
+    Also ``__repr__`` is implemented to make repl-driven development easy :)
+
+    >>> lazyfilter(is_even, range(10))
+    (0, 2, 4, 6, 8)
+
+    """
+
+    def __init__(self, fn: Callable, iterable: Iterable) -> None:
+        self.fn = fn
+        self.iterable = iterable
+
+    def __iter__(self) -> Generator:
+        return (i for i in self.iterable if self.fn(i))
+
+    def __next__(res):
+        return next(res)
+
+    def __repr__(self): # pragma: no cover
+        return str(tuple(self))
+
+
+def filter(fn: Callable, iterable: Iterable) -> Tuple:
+    """filter with eager evaluation.
+    Prefer this over ``lazyfilter``.
+
+    >>> filter(is_even, range(10))
+    (0, 2, 4, 6, 8)
+    """
+    return tuple([i for i in iterable if fn(i)])
+
+
 def flip(fn: Callable) -> Callable:
     """returns a function that takes in a flipped order of args.
     Usage:
@@ -75,6 +180,7 @@ def reduced(x):
     """Use with ``functionali.reduce`` to prematurely terminate ``reduce`` with the value of ``x``.
 
     Usage:
+
     >>> reduce(lambda acc, el: reduced("!"), [1,3,4])
     "!"
     # reduce is prematurely terminated and returns a value of "!"
@@ -163,7 +269,6 @@ def trampoline(fn: Callable, *args: Any):
     >>>        return curr*acc
     >>>    else:
     >>>        return lambda: fact(x, curr+1, acc*curr)
-
     >>> trampoline(fact, 3) == 6
     >>> trampoline(fact, 100000000000) # does not raise RecursionError
     """
